/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 #include "extern.h"

/* -------------------------
	 | ANSI Terminal support |
	 ------------------------- */ 

/* -------------------
	 | Cursor movement |
	 ------------------- */
void ansi_curmove(char *buf)
{
int x, y;
char *s;

if (isdigit(buf[2]))
	y = atoi(buf + 2) - 1;
else
	y = 0;

s = strchr(buf, ';');
if (s)
	x = atoi(s + 1) - 1;
else
	x = 0;

new_crspos(x, y);
}

/* -------------
	 | Cursor up |
	 ------------- */
void ansi_curup(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
cursor_up(n);           
}

/* ---------------
	 | Cursor down |
	 --------------- */
void ansi_curdwn(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;

cursor_down(n);           
}

/* ----------------
	 | Cursor right |
	 ---------------- */
void ansi_currgt(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
cursor_right(n);           
}

/* ---------------
	 | Cursor left |
	 --------------- */
void ansi_curlft(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
cursor_left(n);           
}

/* -------------------
	 | Ansi attributes |
	 ------------------- */
void ansi_setattr(char *buf)
{
extern int obj_colours;

buf += 2;
if (!isdigit(*buf))
	{
	if (obj_colours < 16)
		attr = 0;
	else
		attr = def_foreground | (def_background << 4);
	return;
	}

while(isdigit(*buf))
	{
	switch(atoi(buf))
		{
		case 0:
			if (obj_colours < 16)
				attr = 0;
			else
				attr = def_foreground | (def_background << 4);
			break;

		case 1:
		case 3:
			if (obj_colours < 16)
				attr |= 1;
			else
				attr |= 8;
			break;

		case 2:
			if (obj_colours < 16)
				attr &= ~1;
			else
				attr &= ~0x8;
			break;

		case 4:
			if (obj_colours < 16)
				attr |= 4;
			break;
			
		case 5:
		case 6:
			if (obj_colours >= 16)
				attr |= 0x80;
			break;

		case 7:
			if (obj_colours < 16)
				attr |= 0x80;
			else
				attr = (attr & 0x88) | def_rforeground | (def_rbackground << 4);
			break;

		case 30:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_BLACK;
				}
			break;

		case 31:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_RED;
				}
			break;

		case 32:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_GREEN;
				}
			break;

		case 33:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_BROWN;
				}
			break;

		case 34:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_BLUE;
				}
			break;

		case 35:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_MAGENTA;
				}
			break;

		case 36:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_CYAN;
				}
			break;

		case 37:
			if (obj_colours >= 16)
				{
				attr &= ~0x7;
				attr |= COL_GREY;
				}
			break;

		case 40:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_BLACK << 4;
				}
			break;

		case 41:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_RED << 4;
				}
			break;

		case 42:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_GREEN << 4;
				}
			break;

		case 43:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_BROWN << 4;
				}
			break;

		case 44:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_BLUE << 4;
				}
			break;

		case 45:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_MAGENTA << 4;
				}
			break;

		case 46:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_CYAN << 4;
				}
			break;

		case 47:
			if (obj_colours >= 16)
				{
				attr &= ~0x70;
				attr |= COL_GREY << 4;
				}
			break;
		}

	buf = strchr(buf, ';');
	if (!buf)
		break;
	buf++;
	}
}

/* --------------------
	 | Delete character |
	 -------------------- */
void ansi_delchar(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
del_character(n);           
}

/* ---------------
	 | Delete line |
	 --------------- */
void ansi_delline(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
delete_lines(n);           
}

/* ---------------
	 | Insert char |
	 --------------- */
void ansi_inschar(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
ins_character(n);
}

/* --------------------
	 | Cursor next line |
	 -------------------- */
void ansi_nextline(char *buf)
{
int n;

character_out('\r');
character_out('\n');
n = atoi(buf + 2);
if (n)
	{
	n--;
	while(n)
		character_out('\n');
	}
}

/* --------------------
	 | Go previous line |
	 -------------------- */
void ansi_prevline(char *buf)
{
int n;

character_out('\r');
n = atoi(buf + 2);
if (n)
	cursor_up(n);
else
	cursor_up(1);
}

/* ----------------
	 | Insert lines |
	 ---------------- */
void ansi_insline(char *buf)
{
int n;

n = atoi(buf + 2);
if (!n)
	n = 1;
ins_lines(n);
}

/* ---------------------
	 | Emulate Ansi keys |
	 --------------------- */
void emulate_key(void)
{
switch(key_scan)
	{
	case 0x3B:
	case 0x3C:
	case 0x3D:
	case 0x3E:
	case 0x3F:
	case 0x40:
	case 0x41:
	case 0x42:
	case 0x43:
	case 0x44:
		send_string("\033[");
		if (k_state & KSHIFT)
			 if (key_scan < 0x3F)
				 tcpip_write((char)('W' + key_scan - 0x3B));
			 else
				 tcpip_write((char)('a' + key_scan - 0x3F));
		else
			tcpip_write((char)('M' + key_scan - 0x3B));
		break;

	case 0x47:
		send_string("\033[H");
		break;

	case 0x48:
		send_string("\033[A");
		break;

	case 0x49:
		send_string("\033[I");
		break;

	case 0x4B:
		send_string("\033[D");
		break;

	case 0x4D:
		send_string("\033[C");
		break;

	case 0x4F:
		send_string("\033[F");
		break;

	case 0x50:
		send_string("\033[B");
		break;

	case 0x51:
		send_string("\033[G");
		break;

	case 0x52:
		send_string("\033[L");
		break;

	case 0x53:
		send_string("\177");
		break;

	case 0x57:
	case 0x58:
		send_string("\033[");
		tcpip_write((char)('W' + key_scan - 0x57));
		break;
	}
}
