/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"

int icony_flg;

/* ---------------------
   | Is object checked |
   --------------------- */
int IsChecked(OBJECT *tree, int obj)
{
return (tree[obj].ob_state & CHECKED) &&
			 !(tree[obj].ob_state & DISABLED);
}

/* -------------
   | Set check |
   ------------- */
void CheckObj(OBJECT *tree, int obj)
{
tree[obj].ob_state |= CHECKED;
}

/* ---------------
   | Unset check |
   --------------- */
void UncheckObj(OBJECT *tree, int obj)
{
tree[obj].ob_state &= ~CHECKED;
}

/* ---------------
   | Unset check |
   --------------- */
void ToggleCheck(OBJECT *tree, int obj)
{
if (!(tree[obj].ob_state & DISABLED))
	tree[obj].ob_state ^= CHECKED;
}

/* -----------------
   | Enable object |
   ----------------- */
void EnableObj(OBJECT *tree, int obj)
{
tree[obj].ob_state &= ~DISABLED;
}

/* ------------------
   | Disable object |
   ------------------ */
void DisableObj(OBJECT *tree, int obj)
{
tree[obj].ob_state |= DISABLED;
}

/* ---------------------
   | (De)select object |
   --------------------- */
void SetSelect(OBJECT *tree, int obj, int set)
{
tree[obj].ob_state &= ~SELECTED;
if (set) tree[obj].ob_state |= SELECTED;
}

/* -----------------------
   | Check select status |
   ----------------------- */
int IsSelected(OBJECT *tree, int obj)
{
return tree[obj].ob_state & SELECTED;
}

/* ---------------
   | Draw dialog |
   --------------- */
void open_dialog(OBJECT *tree)
{
int dx, dy, dw, dh;
int fx, fy, fw, fh;

form_center(tree, &fx, &fy, &fw, &fh);

wind_calc(WC_BORDER, ICONIFIER|NAME|MOVER, fx, fy, fw, fh,
					&dx, &dy, &dw, &dh);
	
dialog_wind = wind_create(SMALLER|NAME|MOVER, 0, 0, dw, dh);
wind_set(dialog_wind, WF_NAME, "DR-Telnet");
wind_open(dialog_wind, dx, dy, dw, dh);
icony_flg = FALSE;

do
	{
	which = evnt_multi(MU_MESAG|MU_TIMER,
										 0, 0, 0,
										 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
										 msg, 200, 0, 
										 &mouse_x, &mouse_y, &m_button,
										 &k_state, &k_return, &clicks);

	if (which & MU_MESAG)
		dialog_message(tree, msg);
	
	}while(which & MU_MESAG);
}

/* ---------------------------
   | Redraw the whole dialog |
   --------------------------- */
void redraw_dialog(OBJECT *tree)
{
int wx, wy, ww, wh;

wind_get(dialog_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

tree[0].ob_x = wx;
tree[0].ob_y = wy;
redraw_tree(tree, ROOT, wx, wy, ww, wh);
}

/* ---------------------------
   | Redraw the whole dialog |
   --------------------------- */
void redraw_object(OBJECT *tree, int obj)
{
int wx, wy, ww, wh;

if (dialog_wind == -1) return;

wind_get(dialog_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

tree[0].ob_x = wx;
tree[0].ob_y = wy;
redraw_tree(tree, obj, wx, wy, ww, wh);
}

/* ------------------
   | Execute dialog |
   ------------------ */
int  do_dialog(OBJECT *tree, int s_obj)
{
int obj;
int wx, wy, ww, wh;

obj = x_form_do(tree, s_obj);
if (tree[obj & 0x7FFF].ob_flags & EXIT)
	{
	wind_get(dialog_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

	tree[0].ob_x = wx;
	tree[0].ob_y = wy;
	objc_change(tree, obj & 0x7FFF, 0, wx, wy, ww, wh, 
							tree[obj & 0x7FFF].ob_state & ~SELECTED, FALSE);
	redraw_tree(tree, obj & 0x7FFF, wx, wy, ww, wh);
	}
return obj;		
}

/* ----------------
   | Close dialog |
   ---------------- */
void close_dialog()
{
wind_close(dialog_wind);
wind_delete(dialog_wind);
dialog_wind = -1;
}

/* ------------------------
   | Change object status |
   ------------------------ */
void change_object(OBJECT *tree, int obj, int state)
{
int wx, wy, ww, wh;

wind_get(dialog_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

tree[0].ob_x = wx;
tree[0].ob_y = wy;
objc_change(tree, obj, 0, wx, wy, ww, wh, state, FALSE);
redraw_tree(tree, obj, wx, wy, ww, wh);
} 

/* ----------------------
   | Redraw object tree |
   ---------------------- */
void redraw_tree(OBJECT *tree, int obj, int rx, int ry, int rw, int rh)
{
GRECT t1, t2;
int wx, wy, ww, wh;

t2.g_x = rx;
t2.g_y = ry;
t2.g_w = rw;
t2.g_h = rh;

set_special_objects(tree);
set_edit_specials(tree);

wind_update(BEG_UPDATE);
v_hide_c(vdi_handle);

wind_get(dialog_wind, WF_WORKXYWH, &wx, &wy, &ww, &wh);

wind_get(dialog_wind, WF_FIRSTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
tree[0].ob_x = wx;
tree[0].ob_y = wy;

iconified[0].ob_x = wx;
iconified[0].ob_y = wy;
iconified[0].ob_width = ww;
iconified[0].ob_height = wh;

iconified[1].ob_x = (ww - iconified[1].ob_width) >> 1;
iconified[1].ob_y = (wh - iconified[1].ob_height) >> 1;

while(t1.g_w && t1.g_h)
	{
	if (rc_intersect(&t2, &t1))
		if (icony_flg)
			objc_draw(iconified, 0, MAX_DEPTH, t1.g_x, t1.g_y, t1.g_w, t1.g_h);
		else
			objc_draw(tree, obj, MAX_DEPTH, t1.g_x, t1.g_y, t1.g_w, t1.g_h);

	wind_get(dialog_wind, WF_NEXTXYWH, &t1.g_x, &t1.g_y, &t1.g_w, &t1.g_h);
	}

v_show_c(vdi_handle, 1);
wind_update(END_UPDATE);

release_edit_specials(tree);
release_special_objects(tree);
}

/* --------------------------------
   | Manage dialog window message |
   -------------------------------- */
void dialog_message(OBJECT *tree, int *msg)
{
int d;

switch(msg[0])
	{
	case WM_REDRAW:
		if (msg[3] == dialog_wind)	
			redraw_tree(tree, ROOT, msg[4], msg[5], msg[6], msg[7]);
		break;

	case WM_TOPPED:
	case WM_NEWTOP:
		if (msg[3])
			wind_set(msg[3], WF_TOP);
		break;

	case WM_ICONIFY:
	case WM_ALLICONIFY:
		icony_flg = TRUE;
		wind_set(msg[3], WF_ICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;

	case WM_UNICONIFY:
		icony_flg = FALSE;
		wind_set(msg[3], WF_UNICONIFY, msg[4], msg[5], msg[6], msg[7]);
		break;
		
	case WM_BOTTOMED:
		wind_set(msg[3],WF_BOTTOM);
		break;
		
	case AP_TERM:
	case AC_CLOSE:							/* ? */
		break;
		
	case WM_MOVED:
		if (!icony_flg && msg[3] == dialog_wind)
			{
			if (msg[6] < tree[0].ob_width)	msg[6] = tree[0].ob_width;
			if (msg[7] < tree[0].ob_height)	msg[7] = tree[0].ob_height;
			}
		wind_set(msg[3], WF_CURRXYWH, msg[4], msg[5], msg[6], msg[7]);
		break;
	}

if (msg[3] == dialog_wind)
	wind_get(msg[3], WF_WORKXYWH, &tree[0].ob_x, &tree[0].ob_y, &d, &d);
}

/* -----------------
   | Bind tab page |
   ----------------- */
void BindTab(OBJECT *tree1, int obj1, int new_obj)
{
int i;

if (tree1[obj1].ob_head != -1)
	{
	i = tree1[obj1].ob_head;
	for (;;)
		{
		if (((tree1[i].ob_type & 0xFF) == G_FTEXT || 
				 (tree1[i].ob_type & 0xFF) == G_FBOXTEXT) &&
				 (tree1[i].ob_flags & EDITABLE))
			{
			tree1[i].ob_type |= 0x80;
			tree1[i].ob_flags &= ~EDITABLE;
			}

		if (tree1[i].ob_flags & (0x8000|LASTOB))
			break;
			
		i++;
		}
			
	objc_delete(tree1, tree1[obj1].ob_head);
	}
	
i = new_obj;
for(;;)
	{
	if ((tree1[i].ob_type & 0xFF) == (G_FTEXT|0x80) || 
			(tree1[i].ob_type & 0xFF) == (G_FBOXTEXT|0x80))
		{
		tree1[i].ob_type &= ~0x80;
		tree1[i].ob_flags |= EDITABLE;
		}
	if (tree1[i].ob_flags & (0x8000|LASTOB))
		break;
	i++;
	}
	
objc_add(tree1, obj1, new_obj);
}

/* -------------------------
   | Combine two dialogues |
   ------------------------- */
int CombineDialogs(OBJECT **newtree, OBJECT *tree1, OBJECT *tree2)
{
int elem1, elem2, i;
int x, y, w, h;

elem1 = 0;	while(!(tree1[elem1++].ob_flags & LASTOB));
elem2 = 0;	while(!(tree2[elem2++].ob_flags & LASTOB));

if (*newtree != tree1)
	{
	form_center(tree1, &x, &y, &w, &h);
	*newtree = (OBJECT *)malloc(sizeof(OBJECT) * (elem1 + elem2));
	memcpy(*newtree, tree1, sizeof(OBJECT) * elem1);
	}
else
	*newtree = (OBJECT *)realloc(tree1, sizeof(OBJECT) * (elem1 + elem2));

form_center(tree2, &x, &y, &w, &h);
tree2[0].ob_x = tree2[0].ob_y = 0;

memcpy(*newtree + elem1, tree2, sizeof(OBJECT) * elem2);
(*newtree)[elem1 - 1].ob_flags &= ~LASTOB;
(*newtree)[elem1 - 1].ob_flags |= 0x8000;

i = elem1;
while(elem2--)
	{
	if ((((*newtree)[i].ob_type & 0xFF) == G_FTEXT || 
			 ((*newtree)[i].ob_type & 0xFF) == G_FBOXTEXT) &&
			 ((*newtree)[i].ob_flags & EDITABLE))
		{
		(*newtree)[i].ob_type |= 0x80;
		(*newtree)[i].ob_flags &= ~EDITABLE;
		}

	if ((*newtree)[i].ob_next != -1) (*newtree)[i].ob_next += elem1;
	if ((*newtree)[i].ob_head != -1) (*newtree)[i].ob_head += elem1;
	if ((*newtree)[i].ob_tail != -1) (*newtree)[i].ob_tail += elem1;
	i++;		
	}
	
return elem1;
}

