/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"

static char buffer[50];
static int esc_idx = 0, esc_typ = 0;
static int env_mode = 0;
static char env_var[50], env_cnt;

/* ---------------------------
	 | Execute escape sequence |
	 --------------------------- */
void execute_ansi1(char chr)
{
switch(chr)
	{
	case '5':
		key_mode = SCAN_MODE;
		break;

	case '4':
		key_mode = ASCII_MODE;
		break;

  case 'E':
    env_cnt = 0;
    env_mode = 1;
    break;

  case 'G':
    {
    char *s;

    env_var[env_cnt++] = 0;
    s = getenv(env_var);
    if (s)
		  send_string(s);
    tcpip_write('\r');
    env_mode = 0;
    break;
    }
	}
}

/* -----------------------------------
	 | execute variable ansi sequences |
	 ----------------------------------- */
void execute_ansi2(char chr)
{
int col;

buffer[esc_idx] = 0;

if (buffer[2] == '=')
	{
	col = atoi(buffer + 3);

	switch(chr)
		{
		case 'A':
			def_border = col;
			break;

		case 'C':
			cform = (cform & 0xFF) + col * 256;
			if (strchr(buffer + 3, ';'))
				{
				cform = (cform & 0xFF00) + atoi(strchr(buffer + 3, ';') + 1);
				cursor_off();
				cursor_on();
				}
			break;

		case 'E':
			break;

		case 'F':
			def_foreground = col;
			break;

		case 'G':
			def_background = col;
			break;

		case 'H':
			def_rforeground = col;
			break;

		case 'I':
			def_rbackground = col;
			break;

		case 'J':
		case 'K':
			break;
		}
	}
else
	switch(chr)
		{
		case '`':
			set_hori(atoi(buffer + 2));
			break;

		case 'G':
			set_hori(atoi(buffer + 2));
			break;

		case 'a':
			set_rhori(atoi(buffer + 2));
			break;

		case 'd':
			set_verti(atoi(buffer + 2));
			break;

		case 'e':
			set_rverti(atoi(buffer + 2));
			break;

		case 'E':
			ansi_nextline(buffer);
			break;

		case 'F':
			ansi_prevline(buffer);
			break;

		case 'J':
			switch(buffer[2])
				{
				case '2':
					clear_all();
					new_crspos(0,0);
					break;

				case '1':
					clear_tosos();
					break;

				case '0':
				case 0:
					clear_toeos();
					break;
				}
			break;

		case 'H':
		case 'f':
			ansi_curmove(buffer);
			break;

		case 'A':
			ansi_curup(buffer);
			break;

		case 'B':
			ansi_curdwn(buffer);
			break;

		case 'C':
			ansi_currgt(buffer);
			break;

		case 'D':
			ansi_curlft(buffer);
			break;

		case 's':
			save_curpos();
			break;

		case 'u':
			restore_curpos();
			break;

		case 'K':
			switch(buffer[2])
				{
				case 0:
				case '0':
					clear_eol();
					break;

				case '1':
					clear_sol();
					break;

				case '2':
					clear_line();
					break;
				}
			break;

		case 'm':
			ansi_setattr(buffer);
			break;

		case 'P':
			ansi_delchar(buffer);
			break;

		case 'M':
			ansi_delline(buffer);
			break;

		case '@':
			ansi_inschar(buffer);
			break;

		case 'L':
			ansi_insline(buffer);
			break;

		case 'g':
			character_out((char)atoi(buffer + 2));
			break;

		case 'S':
			scroll_up(atoi(buffer + 2));
			break;

		case 'T':
			scroll_down(atoi(buffer + 2));
			break;

		case 'i':
		  if (buffer[2] == '5')
	  	  local_prt = TO_PRT;

			if (buffer[2] == '4')
				local_prt = TO_SCR;
			break;
		}
}

/* ------------------------------
	 | Check for escape sequences |
	 ------------------------------ */
int check_escape(char chr)
{
int i;

if (monitor)
	return 0;

switch(esc_typ)
	{
	case 0:
		if (chr != 0x1B)
			if (chr != 0x7)
        {
        if (env_mode)
          {
          env_var[env_cnt++] = chr;
          if (env_cnt == 20) env_mode = 0;
          return 1;
          }
				return 0;
        }
			else
				{
				visible_bell();
				return 1;
				}
		esc_idx = 0;
		esc_typ = 1;
		break;

	case 1:
		if (chr == '~')
			esc_typ = 2;
		else
			if (chr == '[')
				esc_typ = 3;
			else
				{
				if (local_prt == TO_PRT)
					character_out(27);

				esc_typ = 0;
				return 0;
				}
		break;

	case 2:
		execute_ansi1(chr);
		esc_typ = 0;
		return 1;

	case 3:
		if (!isdigit(chr) && chr != ';' && chr != '=')
			{
			execute_ansi2(chr);
			esc_typ = 0;
			return 1;
			}
		break;
	}

buffer[esc_idx++] = chr;
if (local_prt == TO_PRT)
	{
	if ((chr != '[' && esc_idx == 2) || (chr != '4' && esc_idx == 3))
		{
		for (i = 0; i < esc_idx; i++)
			character_out(buffer[i]);
		esc_typ = 0;
		return 1;
		}
	}
return 1;
}
