/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"

extern unsigned char crt_char[26][80];

void show_mark(void)
{
int xy[4];
int wx, wy, ww, wh;

wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);

xy[0] = wx + marktop_x * font_width;
xy[1] = wy + marktop_y * font_height;
xy[2] = wx + markbot_x * font_width + font_width - 1;
xy[3] = wy + markbot_y * font_height + font_height -1;

vswr_mode(vdi_handle, MD_XOR);
graf_mouse(M_OFF, 0L);
vsf_color(vdi_handle, 1);
vsf_interior(vdi_handle, FIS_SOLID);
vr_recfl(vdi_handle, xy);
graf_mouse(M_ON, 0L);
vswr_mode(vdi_handle, MD_REPLACE);
}

/* ---------------
   | Mark window |
   --------------- */
void mark_window(void)
{
int wx, wy, ww, wh;
int mark_x, mark_y, curr_x, curr_y, old_x, old_y;

if (crt_window == -1) return;

if (wind_find(mouse_x, mouse_y) != crt_window)
	return;

wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);

if (mouse_x < wx || mouse_y < wy ||
		mouse_x >= wx + ww || mouse_y >= wy + wh)
	return;

wind_update(BEG_MCTRL);
	
mark_x = (mouse_x - wx) / font_width;
mark_y = (mouse_y - wy) / font_height;

graf_mkstate(&mouse_x, &mouse_y, &m_button, &k_state);

if (mark_x >= 0 && mark_y >= 0 &&
	  mark_x < 80 && mark_y < 25 && (m_button & 1))
	{
	if (marktop_x != -1)
		show_mark();
		
	if (k_state & 3 && marktop_x != -1)
		{
		if (mark_x < marktop_x || mark_y < marktop_y)
			{
			marktop_x = mark_x;
			marktop_y = mark_y;
			}
		else
			{
			markbot_x = mark_x;
			markbot_y = mark_y;
			}
		}
	else
		{	
		marktop_x = markbot_x = mark_x;
		marktop_y = markbot_y = mark_y;
		}
		
	show_mark();

	old_x = mark_x;
	old_y = mark_y;
	do
		{
		graf_mkstate(&mouse_x, &mouse_y, &m_button, &k_state);

		curr_x = (mouse_x - wx) / font_width;
		curr_y = (mouse_y - wy) / font_height;
		if (curr_x >= 0 && curr_y >= 0 &&
	  		curr_x < 80 && curr_y < 25 &&
	  		(curr_x != old_x || curr_y != old_y))
			{
			show_mark();
			if (curr_x < mark_x || curr_y < mark_y)
				{
				marktop_x = curr_x;
				marktop_y = curr_y;
				markbot_x = mark_x;
				markbot_y = mark_y;
				}
			else
				{
				marktop_x = mark_x;
				marktop_y = mark_y;
				markbot_x = curr_x;
				markbot_y = curr_y;
				}
			show_mark();
			}
		
		old_x = curr_x;
		old_y = curr_y;
		}while(m_button & 1);

	}
wind_update(END_MCTRL);

menu_ienable(menu_adr, M_COPY, marktop_x != -1);
menu_ienable(menu_adr, M_DESEL, marktop_x != -1);
}

/* -------------------
   | Mark everything |
   ------------------- */
void mark_all(void)
{
if (marktop_x != -1)
	show_mark();
	
marktop_x = marktop_y = 0;
markbot_x = 79;
markbot_y = 24;
show_mark();
menu_ienable(menu_adr, M_COPY, TRUE);
menu_ienable(menu_adr, M_DESEL, TRUE);
}

/* --------------------
   | Deselect marking |
   -------------------- */
void unmark_all(void)
{
if (marktop_x != -1)
	show_mark();
	
marktop_x = marktop_y = 
markbot_x = markbot_y = -1;
menu_ienable(menu_adr, M_COPY, FALSE);
menu_ienable(menu_adr, M_DESEL, FALSE);
}

/* ----------------------------------
   | Copy current marked characters |
   ---------------------------------- */
void copy_mark(void)
{
char scrap[256];
int i;
long fh;

if (scrp_read(scrap))
	{
	strcat(scrap, "SCRAP.TXT");

	fh = Fcreate(scrap, 0);
	if (fh > 0)
		{
		for (i = marktop_y; i <= markbot_y; i++)
			{
			Fwrite((int)fh, markbot_x - marktop_x + 1, crt_char[i] + marktop_x);
			if (i < markbot_y)
				Fwrite((int)fh, 2, "\r\n");
			}
			
		Fclose((int)fh);
		menu_ienable(menu_adr, M_PASTE, TRUE);
		}
	else
		form_alert(1, scrap_error);
	}

unmark_all();
}
