/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"

#define MagX_COOKIE 'MagX'
#define MiNT_COOKIE 'MiNT'

int min(int a, int b)	{	return ((a < b) ? a : b);	}
int max(int a, int b)	{	return ((a > b) ? a : b);	}

/* -----------------------------
   | Check for cookie presence |
   ----------------------------- */
long check_cookie(long cook)
{
long old_stk, *old_jar;

old_stk = Super(0L);

old_jar = *(long **)0x5a0;
Super((void *)old_stk);

if (!old_jar)
	return 0L;
	
while(*old_jar)
	{
	if (*old_jar == cook)
		return old_jar[1];
		
	old_jar += 2;
	}

return 0L;
}

/* ------------------------
   | Check current system |
   ------------------------ */
int CheckSystem(void)
{
int sys;

sys = 0;
if (check_cookie(MiNT_COOKIE))
	{
  if (_GemParBlk.global[1] > 1 || _GemParBlk.global[1] == -1)
    sys = 3;
  else
  	sys = 1;
	}

if (check_cookie(MagX_COOKIE))
  sys = 2;
  
return sys;
}

/* ------------------------------------
   | Calculate rectangle intersection |
   ------------------------------------ */
int rc_intersect(GRECT *s1, GRECT *s2)
{
int tx, ty, tw, th;

tw = min(s2->g_x + s2->g_w, s1->g_x + s1->g_w);
th = min(s2->g_y + s2->g_h, s1->g_y + s1->g_h);
tx = s2->g_x = max(s2->g_x, s1->g_x);
ty = s2->g_y = max(s2->g_y, s1->g_y);
s2->g_w = tw - tx;
s2->g_h = th - ty;

return ((tw > tx) && (th > ty));
}

/* -----------------
	 | Send a string |
	 ----------------- */
void send_string(char *sstr)
{
while(*sstr)
	tcpip_write(*(sstr++));
}

/* -------------------------------------
   | Check current state of connection |
   ------------------------------------- */
int check_connection(int function)
{
switch (get_connected())
	{
	case -1:
		form_alert(1, nodriver_msg);
		return FALSE;
	
	case 0:
		{
		int draconis_id;
		
		draconis_id = appl_find("DRACONIS");
		if (draconis_id < 0)
			form_alert(1, offline_msg);
		else
			{
			int msg[8];
			
			msg[0] = 0x5811;
			msg[1] = gl_apid;
			msg[3] = function;
			appl_write(draconis_id, 16, msg);
			}
		return FALSE;
		}
		
	case 1:
		while (get_connected() == 1)
		  if (get_connected() != 2 && get_connected() != 1)
			  return FALSE;
			
	case 2:
		return TRUE;
	}
return FALSE;
}
