/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"

static int crt_x, store_x;
static int crt_y, store_y;
static int last_crt_x = -1, last_crt_y, flush_x = -1;
unsigned char crt_attr[26][80], crt_char[26][80], flush_attr;
static int cursor_flag, cursor_cnt;
extern int obj_colours;

/* --------------------------
	 | Initialize screen data |
	 -------------------------- */
void crt_init(void)
{
int wx, wy, ww, wh;
int ax, ay, aw, ah;
int d;

if (crt_window != -1) crt_exit();

find_font();

if (obj_colours < 16)
	attr = 0;
else
	attr = (def_background << 4) | def_foreground;
memset(crt_attr, attr, sizeof(crt_attr));
memset(crt_char, ' ', sizeof(crt_char));
crt_x = 0;
crt_y = 0;
cursor_flag = 0;
local_prt = TO_SCR;

wind_get(0, WF_WORKXYWH, &wx, &wy, &ww, &wh);
crt_window = wind_create(NAME|CLOSER|FULLER|MOVER,
												 wx, wy, 80 * font_width, 25 * font_height);

wind_set(crt_window, WF_NAME, hostname);

wind_calc(WC_BORDER, NAME|CLOSER|FULLER|MOVER, 
					0, 0, 80 * font_width, 25 * font_height, &ax, &ay, &aw, &ah);
					
wind_open(crt_window, wx, wy, aw, ah);

vsf_interior(vdi_handle, FIS_SOLID);
vst_alignment(vdi_handle, 0, 5, &d, &d);

check_scrap();
menu_ienable(menu_adr, M_SELALL, TRUE);
}

/* --------------------
   | Close CRT screen |
   -------------------- */
void crt_exit(void)
{
if (crt_window != -1)
	{
	wind_close(crt_window);
	wind_delete(crt_window);
	crt_window = -1;

	menu_ienable(menu_adr, M_COPY, 		FALSE);
	menu_ienable(menu_adr, M_PASTE, 	FALSE);
	menu_ienable(menu_adr, M_SELALL, 	FALSE);
	menu_ienable(menu_adr, M_DESEL, 	FALSE);
	}
}

/* ---------------------------
   | Flush buffer crt output |
   --------------------------- */
void CrtFlush(void)
{
int rx, ry, rw, rh;
int wx, wy, ww, wh;
int xy[4], rxy[4], reverse;
unsigned char c;
    
if (flush_x == -1) return;

cursor_off();

if (obj_colours < 16)
	{
	vst_effects(vdi_handle, flush_attr & 0x7F);
	if (flush_attr & 0x80)
		{
		reverse = 1;
		vst_color(vdi_handle, 1);
		vsf_color(vdi_handle, 1);
		}
	else
		{
		reverse = 0;
		vst_color(vdi_handle, 1);
		vsf_color(vdi_handle, 0);
		}
	}
else
	{
	reverse = 0;
	vst_color(vdi_handle, color_map[flush_attr & 0xF]);
	vsf_color(vdi_handle, color_map[flush_attr >> 4]);
	}
	
wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);
  	
rxy[0] = wx + font_width * flush_x;
rxy[1] = wy + font_height * crt_y;
rxy[2] = rxy[0] + (crt_x - flush_x) * font_width - 1;
rxy[3] = rxy[1] + font_height - 1;
  	
wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);
wind_get(crt_window, WF_FIRSTXYWH, &rx, &ry, &rw, &rh);
while (rw && rh)
	{
	xy[0] = rx; 
	xy[1] = ry;
	xy[2] = rx + rw - 1;
	xy[3] = ry + rh - 1;
  		
	vs_clip(vdi_handle, 1, xy);

	vswr_mode(vdi_handle, MD_REPLACE);
	vr_recfl(vdi_handle, rxy);

	vswr_mode(vdi_handle, reverse ? MD_XOR : MD_TRANS);
	c = crt_char[crt_y][crt_x];
	crt_char[crt_y][crt_x] = 0;
	v_gtext(vdi_handle, rxy[0], rxy[1], (char *)&crt_char[crt_y][flush_x]);
	crt_char[crt_y][crt_x] = c;
  		
	vs_clip(vdi_handle, 0, xy);
  		
 	wind_get(crt_window, WF_NEXTXYWH, &rx, &ry, &rw, &rh);
	}
graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);

flush_x = -1;
}

/* --------------------
   | Character output |
   -------------------- */
void character_out(char chr)
{
if (local_prt == TO_PRT)
	{
	Cprnout(chr);
	return;
	}

switch(chr)
	{
	case '\t':
  	do
	  	{
      if (crt_x < 79)
		    cursor_right(1);
      else
        {
  			if (crt_y == 24)
          scroll_up(1);
		  	else
		  		{
		  		CrtFlush();
		  		cursor_off();
			  	crt_y++;
			  	}

			  crt_x = 0;
        }
		  }while(crt_x & 7);
      break;

  case 7:
    Cconws("\007");
    break;

	case 8:
    cursor_left(1);
    break;

	case 13:
    new_crspos(0, crt_y);
    break;

  case 10:
		if (chr == 0xA && crt_y == 24)
      scroll_up(1);
		else
      new_crspos(crt_x, crt_y + 1);
	break;

	default:
    {
    if (attr != flush_attr || flush_x == -1)
    	{
   		CrtFlush();
   		cursor_off();
    	flush_x = crt_x;
    	flush_attr = attr;
    	}
    	
    crt_attr[crt_y][crt_x] = attr;
    crt_char[crt_y][crt_x] = chr;
  
		crt_x++;
		if (crt_x == 80)
			{
			if (crt_y == 24)
        scroll_up(1);
			else
				{
				CrtFlush();
	  		cursor_off();
				crt_y++;
				}

			crt_x = 0;
			}
		return;
    }
	}
}

/* -----------------------
   | Flush screen buffer |
   ----------------------- */
void FlushScreen(void)
{
CrtFlush();
if (last_crt_x != crt_x || last_crt_y != crt_y)
  set_crspos(crt_x, crt_y);
}

/* -----------------------
	 | Set cursor position |
	 ----------------------- */
void set_crspos(int x, int y)
{
/* cursor_off(); */
CrtFlush();

last_crt_x = x;
last_crt_y = y;
/* cursor_on(); */
}

/* ------------------------------
	 | Create new cursor position |
	 ------------------------------ */
void new_crspos(int x, int y)
{
CrtFlush();
cursor_off();
crt_x = x;
crt_y = y;
}

/* ----------------
	 | Clear screen |
	 ---------------- */
void clear_all(void)
{
CrtFlush();
cursor_off();

memset(crt_attr, attr, sizeof(crt_attr));
memset(crt_char, ' ', sizeof(crt_char));

if (obj_colours < 16)
	fill_area(0, 0, 80 * font_width, 25 * font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, 0, 80 * font_width, 25 * font_height, color_map[attr >> 4]);
new_crspos(0, 0);
}

/* -------------------------
	 | Delete (n) characters |
	 ------------------------- */
void del_character(int n)
{
int nx;

CrtFlush();
cursor_off();

if (crt_x + n >= 80)	n = 80 - crt_x;

if (80 - crt_x - n)
	{
	memmove(&crt_attr[crt_y][crt_x], &crt_attr[crt_y][crt_x + n],
					80 - crt_x - n);
	memmove(&crt_char[crt_y][crt_x], &crt_char[crt_y][crt_x + n],
					80 - crt_x - n);

	rect_blt((crt_x + n) * font_width, crt_y * font_height, 
						crt_x * font_width, crt_y * font_height, 
					 (80 - crt_x - n) * font_width, font_height);
	}

nx = 80 - n;
memset(&crt_attr[crt_y][nx], attr, n);
memset(&crt_char[crt_y][nx], ' ', n);

if (obj_colours < 16)
	fill_area(nx * font_width, crt_y * font_height, 
						n * font_width, font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(nx * font_width, crt_y * font_height, 
						n * font_width, font_height, color_map[attr >> 4]);
}

/* ------------------
	 | Delete n lines |
	 ------------------ */
void delete_lines(int n)
{
CrtFlush();
cursor_off();

if (crt_y + n > 25)
	n = 25 - crt_y;

if (crt_y + n < 25)
	{
	memmove(&crt_attr[crt_y][0], &crt_attr[crt_y + n][0], 25 * 80 - (n + crt_y) * 80);
	memmove(&crt_char[crt_y][0], &crt_char[crt_y + n][0], 25 * 80 - (n + crt_y) * 80);
	}

memset(&crt_attr[25 - n][0], attr, n * 80);
memset(&crt_char[25 - n][0], ' ', n * 80);

rect_blt(0, (crt_y + n) * font_height, 0, crt_y * font_height, 80 * font_width, (25 - n - crt_y) * font_height);
if (obj_colours < 16)
	fill_area(0, (25 - n) * font_height, 80 * font_width, n * font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, (25 - n) * font_height, 80 * font_width, n * font_height, color_map[attr >> 4]);
}

/* ------------------------
	 | Save cursor position |
	 ------------------------  */
void save_curpos(void)
{
store_x = crt_x;
store_y = crt_y;
}

/* ---------------------------
	 | Restore cursor position |
	 --------------------------- */
void restore_curpos(void)
{
CrtFlush();
new_crspos(store_x, store_y);
}

/* ----------------
	 | Visible bell |
	 ---------------- */
void visible_bell(void)
{
character_out(0x7);
}

/* ----------------
	 | Clear to eos |
	 ---------------- */
void clear_toeos(void)
{
CrtFlush();
cursor_off();

memset(&crt_char[0][crt_x], ' ', 80 - crt_x + (24 - crt_y) * 80);
memset(&crt_attr[0][crt_x], attr, 80 - crt_x + (24 - crt_y) * 80);

if (obj_colours < 16)
	{
	fill_area(crt_x * font_width, crt_y * font_height, 
						(80 - crt_x) * font_width, font_height, (attr & 0x80) ? 1 : 0);
	if (crt_y < 24)
		fill_area(0, (crt_y + 1) * font_height, 
							80 * font_width, (24 - crt_y) * font_height, (attr & 0x80) ? 1 : 0);
	}
else
	{
	fill_area(crt_x * font_width, crt_y * font_height, 
						(80 - crt_x) * font_width, font_height, color_map[attr >> 4]);
	if (crt_y < 24)
		fill_area(0, (crt_y + 1) * font_height, 80 * font_width, 
							(24 - crt_y) * font_height, color_map[attr >> 4]);
	}
}

/* ----------------
	 | Clear to eol |
	 ---------------- */
void clear_eol(void)
{
CrtFlush();
cursor_off();

memset(&crt_char[0][crt_x], ' ', 80 - crt_x);
memset(&crt_attr[0][crt_x], attr, 80 - crt_x);

if (obj_colours < 16)
	fill_area(crt_x * font_width, crt_y * font_height, 
						(80 - crt_x) * font_width, font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(crt_x * font_width, crt_y * font_height, 
						(80 - crt_x) * font_width, font_height, color_map[attr >> 4]);
}

/* -----------------------
	 | Cursor down n times |
	 ----------------------- */
void cursor_down(int n)
{
CrtFlush();
cursor_off();

if (crt_y + n <= 24)
	new_crspos(crt_x, crt_y + n);
}

/* ---------------------
	 | Cursor up n times |
	 --------------------- */
void cursor_up(int n)
{
CrtFlush();
cursor_off();

if (crt_y - n >= 0)
	new_crspos(crt_x, crt_y - n);
}

/* -----------------------
	 | Cursor left n times |
	 ----------------------- */
void cursor_left(int n)
{
CrtFlush();
cursor_off();

if (crt_x - n >= 0)
	new_crspos(crt_x - n, crt_y);
}

/* ------------------------
	 | Cursor right n times |
	 ------------------------ */
void cursor_right(int n)
{
CrtFlush();
cursor_off();

if (crt_x + n <= 79)
	new_crspos(crt_x + n, crt_y);
}

/* -----------------------
	 | Make cursor visible |
	 ----------------------- */
void cursor_on(void)
{
if (!cursor_flag)
	{
	toggle_cursor();
	cursor_flag = 1;
	}
}

/* -------------------------
	 | Make cursor invisible |
	 ------------------------- */
void cursor_off(void)
{
if (cursor_flag)
	{
	toggle_cursor();
	cursor_flag = 0;
	}
}

/* -----------------------
	 | Insert n lines line |
	 ----------------------- */
void ins_lines(int n)
{
CrtFlush();
cursor_off();

if (crt_y + n > 25)
	n = 25 - crt_y;
	
memmove(&crt_attr[crt_y + n][0], &crt_attr[crt_y][0], 25 * 80 - n * 80);
memmove(&crt_char[crt_y + n][0], &crt_char[crt_y][0], 25 * 80 - n * 80);
memset(&crt_attr[crt_y][0], attr, n * 80);
memset(&crt_char[crt_y][0], ' ', n * 80);

rect_blt(0, crt_y * font_height, 
				 0, (crt_y + n) * font_height, 80 * font_width, 
				 (25 - n - crt_y) * font_height);
if (obj_colours < 16)
	fill_area(0, crt_y * font_height, 
						80 * font_width, n * font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, crt_y * font_height, 
						80 * font_width, n * font_height, color_map[attr >> 4]);
}

/* -----------------------
	 | Insert n characters |
	 ----------------------- */
void ins_character(int n)
{
CrtFlush();
cursor_off();

if (crt_x + n > 80)
	n = 80 - crt_x;
	
if (crt_x + n < 80)
	{
	memmove(&crt_attr[crt_y][crt_x + n], &crt_attr[crt_y][crt_x],
					80 - crt_x -n);
	memmove(&crt_char[crt_y][crt_x + n], &crt_char[crt_y][crt_x],
					80 - crt_x -n);
	}

memset(&crt_attr[crt_y][crt_x], attr, n);
memset(&crt_char[crt_y][crt_x], ' ', n);

rect_blt(crt_x * font_width, crt_y * font_height, 
				(crt_x + n) * font_width, crt_y * font_height, n * font_width, font_height);
if (obj_colours < 16)
	fill_area(crt_x * font_width, crt_y * font_height, 
						n * font_width, font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(crt_x * font_width, crt_y * font_height, 
						n * font_width, font_height, color_map[attr >> 4]);
}

/* ------------------
	 | Set horizontal |
	 ------------------ */
void set_hori(int n)
{
CrtFlush();
cursor_off();

new_crspos(n, crt_y);
}

/* ---------------------------
	 | Set relative horizontal |
	 --------------------------- */
void set_rhori(int n)
{
CrtFlush();

new_crspos(max(min(79, crt_x + n), 0), crt_y);
}

/* ------------------
	 | Set horizontal |
	 ------------------ */
void set_verti(int n)
{
CrtFlush();
cursor_off();

new_crspos(crt_x, n);
}

/* ---------------------------
	 | Set relative horizontal |
	 --------------------------- */
void set_rverti(int n)
{
CrtFlush();
cursor_off();

new_crspos(crt_x, max(min(24, crt_y + n), 0));
}

/* ----------------------------
	 | Clear to start of screen |
	 ---------------------------- */
void clear_tosos(void)
{
CrtFlush();
cursor_off();

memset(&crt_char[0][0], ' ', crt_x + crt_y * 80);
memset(&crt_attr[0][0], attr, crt_x + crt_y * 80);

if (obj_colours < 16)
	{
	if (crt_y)
		fill_area(0, 0, 80 * font_width, 
							crt_y * font_height, (attr & 0x80) ? 1 : 0);
	fill_area(0, crt_y * font_height, crt_x * font_width, 
						font_height, (attr & 0x80) ? 1 : 0);
	}
else
	{
	if (crt_y)
		fill_area(0, 0, 80 * font_width, 
							crt_y * font_height, color_map[attr >> 4]);
	fill_area(0, crt_y * font_height, crt_x * font_width, 
						font_height, color_map[attr >> 4]);
	}
}

/* --------------------------
	 | Clear to start of line |
	 -------------------------- */
void clear_sol(void)
{
CrtFlush();
cursor_off();

memset(&crt_char[crt_y][0], ' ', crt_x);
memset(&crt_attr[crt_y][0], attr, crt_x);

if (obj_colours < 16)
	fill_area(0, crt_y * font_height, crt_x * font_width, 
						font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, crt_y * font_height, crt_x * font_width, 
						font_height, color_map[attr >> 4]);
}

/* --------------
	 | Clear line |
	 -------------- */
void clear_line(void)
{
CrtFlush();
cursor_off();

memset(&crt_char[crt_y][0], ' ', 80);
memset(&crt_attr[crt_y][0], attr, 80);

if (obj_colours < 16)
	fill_area(0, crt_y * font_height, 
						80 * font_width, font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, crt_y * font_height, 
						80 * font_width, font_height, color_map[attr >> 4]);
}

/* ------------------
	 | Print a string |
	 ------------------ */
void string_out(char *sstr)
{
while(*sstr)
  character_out(*(sstr++));
}

/* ---------------------
	 | Scroll up n lines |
	 --------------------- */
void scroll_up(int n)
{
CrtFlush();
cursor_off();

memmove(&crt_attr[0][0], &crt_attr[n][0], 25 * 80 - n * 80);
memmove(&crt_char[0][0], &crt_char[n][0], 25 * 80 - n * 80);
memset(&crt_attr[25 - n][0], attr, n * 80);
memset(&crt_char[25 - n][0], ' ', n * 80);

rect_blt(0, n * font_height, 0, 0, 80 * font_width, (25 - n) * font_height);
if (obj_colours < 16)
	fill_area(0, (25 - n) * font_height, 80 * font_width, 
						n * font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, (25 - n) * font_height, 80 * font_width, 
						n * font_height, color_map[attr >> 4]);
}

/* -----------------------
	 | Scroll down n lines |
	 ----------------------- */
void scroll_down(int n)
{
CrtFlush();
cursor_off();

memmove(&crt_attr[n][0], &crt_attr[0][0], 25 * 80 - n * 80);
memmove(&crt_char[n][0], &crt_char[0][0], 25 * 80 - n * 80);
memset(&crt_attr[0][0], attr, n * 80);
memset(&crt_char[0][0], ' ', n * 80);

rect_blt(0, 0, 0, n * font_height, 80 * font_width, (25 - n) * font_height);
if (obj_colours < 16)
	fill_area(0, 0, 80 * font_width, n * font_height, (attr & 0x80) ? 1 : 0);
else
	fill_area(0, 0, 80 * font_width, n * font_height, color_map[attr >> 4]);
}

/* --------------------
   | Rectangle bitblt |
   -------------------- */
void rect_blt(int xf, int yf, int xt, int yt, int w, int h)
{
int c_xy[4];
int wx, wy, ww, wh;
int rx, ry, rw, rh;

wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);

wind_update(BEG_UPDATE);
wind_get(crt_window, WF_FIRSTXYWH, &rx, &ry, &rw, &rh);
while(rw && rh)
	{
	c_xy[0] = rx;
	c_xy[1] = ry;
	c_xy[2] = rx + rw - 1;
	c_xy[3] = ry + rh - 1;

	vs_clip(vdi_handle, 1, c_xy);
	blt_copy(wx + xf, wy + yf, wx + xt, wy + yt, w, h);
	vs_clip(vdi_handle, 0, c_xy);

	wind_get(crt_window, WF_NEXTXYWH, &rx, &ry, &rw, &rh);
	}
wind_update(END_UPDATE);
}

/* ---------------------------------
   | Bitblit from screen/to screen |
   --------------------------------- */
void blt_copy(int xf, int yf, int xt, int yt, int w, int h)
{
MFDB src, des;
int xy[8];

memset(&src, 0, sizeof(MFDB));
memset(&des, 0, sizeof(MFDB));

xy[0] = xf;
xy[1] = yf;
xy[2] = xf + w - 1;
xy[3] = yf + h - 1;
xy[4] = xt;
xy[5] = yt;
xy[6] = xt + w - 1;
xy[7] = yt + h - 1;

graf_mouse(M_OFF, 0L);
vro_cpyfm(vdi_handle, S_ONLY, xy, &src, &des);
graf_mouse(M_ON, 0L);
}

/* -------------------------
   | Fill area with colour |
   ------------------------- */
void fill_area(int x, int y, int w, int h, int col)
{
int xy[4];
int c_xy[4];
int wx, wy, ww, wh;
int rx, ry, rw, rh;

vswr_mode(vdi_handle, MD_REPLACE);
vsf_color(vdi_handle, col);

wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);

xy[0] = wx + x; 
xy[1] = wy + y;
xy[2] = wx + x + w - 1;
xy[3] = wy + y + h - 1;

graf_mouse(M_OFF, 0L);
wind_update(BEG_UPDATE);
wind_get(crt_window, WF_FIRSTXYWH, &rx, &ry, &rw, &rh);
while(rw && rh)
	{
	c_xy[0] = rx;
	c_xy[1] = ry;
	c_xy[2] = rx + rw - 1;
	c_xy[3] = ry + rh - 1;

	vs_clip(vdi_handle, 1, c_xy);
	vr_recfl(vdi_handle, xy);
	vs_clip(vdi_handle, 0, c_xy);

	wind_get(crt_window, WF_NEXTXYWH, &rx, &ry, &rw, &rh);
	}
wind_update(END_UPDATE);
graf_mouse(M_ON, 0L);
}

/* -----------------
   | Draw all data |
   ----------------- */
void draw_line(int wx, int wy, unsigned char *att, unsigned char *chr)
{
int idx, cnt;
int xy[4], reverse;
unsigned char c, *chr2;

xy[0] = wx;
xy[1] = wy;
xy[3] = wy + font_height - 1;

idx = 80;
while (idx)
	{
	cnt = 1;
	while(att[cnt] == *att && cnt < idx)
		cnt++;

	vswr_mode(vdi_handle, MD_REPLACE);
	if (obj_colours < 16)
		{
		vst_effects(vdi_handle, *att & 0x7F);
		if (*att & 0x80)
			{
			vst_color(vdi_handle, 1);
			vsf_color(vdi_handle, 1);
			reverse = 1;
			}
		else
			{
			vst_color(vdi_handle, 1);
			vsf_color(vdi_handle, 0);
			reverse = 0;
			}
		}
	else
		{
		vsf_color(vdi_handle, color_map[(*att) >> 4]);
		reverse = 0;
		}
		
	xy[2] = xy[0] + cnt * 8;
	vr_recfl(vdi_handle, xy);
	
	chr2 = chr + cnt;
	c = *chr2;
	*chr2 = 0;
	vswr_mode(vdi_handle, reverse ? MD_XOR : MD_TRANS);
	v_gtext(vdi_handle, wx, wy, (char *)chr);
	*chr2 = c;
	
	chr = chr2;
	xy[0] = wx = xy[2];

	att += cnt;
	idx -= cnt;
	}
}

/* -----------------
   | Draw all data |
   ----------------- */
void draw_all(int wx, int wy)
{
int i;

for (i = 0; i < 25; i++)
	draw_line(wx, wy + i * font_height, &crt_attr[i][0], &crt_char[i][0]);
}

/* ---------------------
   | Redraw crt window |
   --------------------- */
void redraw_crt(int rdx, int rdy, int rdw, int rdh)
{
int c_xy[4];
int wx, wy, ww, wh;
GRECT r1, r2;

cursor_off();

wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);

r1.g_x = rdx;
r1.g_y = rdy;
r1.g_w = rdw;
r1.g_h = rdh;

graf_mouse(M_OFF, 0L);
wind_update(BEG_UPDATE);
wind_get(crt_window, WF_FIRSTXYWH, &r2.g_x, &r2.g_y, &r2.g_w, &r2.g_h);
while(r2.g_w && r2.g_h)
	{
	if (rc_intersect(&r1, &r2))
		{
		c_xy[0] = r2.g_x;
		c_xy[1] = r2.g_y;
		c_xy[2] = r2.g_x + r2.g_w - 1;
		c_xy[3] = r2.g_y + r2.g_h - 1;
	
		vs_clip(vdi_handle, 1, c_xy);
		
		draw_all(wx, wy);
				
		vs_clip(vdi_handle, 0, c_xy);
		}
		
	wind_get(crt_window, WF_NEXTXYWH, &r2.g_x, &r2.g_y, &r2.g_w, &r2.g_h);
	}
wind_update(END_UPDATE);
graf_mouse(M_ON, 0L);
}

/* ----------------------
   | Turn cursor on/off |
   ---------------------- */
void toggle_cursor(void)
{
int xy[4];
int c_xy[4];
int wx, wy, ww, wh;
int rx, ry, rw, rh;

vswr_mode(vdi_handle, MD_XOR);
if (obj_colours < 16)
	vsf_color(vdi_handle, 1);
else
	vsf_color(vdi_handle, color_map[7]);

wind_get(crt_window, WF_WORKXYWH, &wx, &wy, &ww, &wh);

xy[0] = wx + crt_x * font_width; 
xy[3] = wy + crt_y * font_height + font_height - 1;
xy[1] = xy[3] - font_height / 4;
xy[2] = xy[0] + font_width - 1;

wind_update(BEG_UPDATE);
graf_mouse(M_OFF, 0L);
wind_get(crt_window, WF_FIRSTXYWH, &rx, &ry, &rw, &rh);
while(rw && rh)
	{
	c_xy[0] = rx;
	c_xy[1] = ry;
	c_xy[2] = rx + rw - 1;
	c_xy[3] = ry + rh - 1;

	vs_clip(vdi_handle, 1, c_xy);
	vr_recfl(vdi_handle, xy);
	vs_clip(vdi_handle, 0, c_xy);

	wind_get(crt_window, WF_NEXTXYWH, &rx, &ry, &rw, &rh);
	}
graf_mouse(M_ON, 0L);
wind_update(END_UPDATE);
cursor_cnt = 5*3;
}

/* -------------------------
   | Cursor blink function |
   ------------------------- */
void cursor_blink(void)
{
if (!cursor_cnt--)
	{
	toggle_cursor();
	cursor_flag ^= 1;
	}
}
