/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"

/* -------------------
   | Set option page |
   ------------------- */
void SetOptPage(OBJECT *tree, int page, int obj)
{
static int tabids[] = {OPTS2, OPTS1};
int i;

BindTab(tree, OPTBKG, page);
redraw_object(tree, OPTBKG);

for (i = 0; i < 2; i++)
	{
	SetSelect(tree, tabids[i], obj == tabids[i]);
	redraw_object(tree, tabids[i]);
	}
redraw_object(tree, obj);
}

/* ------------------------
   | Set current settings |
   ------------------------ */
void get_dialog_settings(void)
{
local_echo = IsSelected(option_tree, opt_pages[0] + L_ECHO);
cut_128 = IsSelected(option_tree, opt_pages[0] + CUT128);
monitor = IsSelected(option_tree, opt_pages[0] + M_MODE);
key_mode = IsSelected(option_tree, opt_pages[0] + KB_ASCII) ? ASCII_MODE : SCAN_MODE;

def_foreground  = back_map[option_tree[opt_pages[1] + N_FORE].ob_spec.obspec.interiorcol];
def_background  = back_map[option_tree[opt_pages[1] + N_BACK].ob_spec.obspec.interiorcol];
def_rforeground = back_map[option_tree[opt_pages[1] + R_BACK].ob_spec.obspec.interiorcol];
def_rbackground = back_map[option_tree[opt_pages[1] + R_FORE].ob_spec.obspec.interiorcol];
}

/* ------------------------
   | Set current settings |
   ------------------------ */
void set_dialog_settings(void)
{
SetSelect(option_tree, opt_pages[0] + L_ECHO, 	local_echo);
SetSelect(option_tree, opt_pages[0] + CUT128, 	cut_128);
SetSelect(option_tree, opt_pages[0] + M_MODE, 	monitor);
SetSelect(option_tree, opt_pages[0] + KB_ASCII, key_mode == ASCII_MODE);
SetSelect(option_tree, opt_pages[0] + KB_SCAN, 	key_mode == SCAN_MODE);

option_tree[opt_pages[1] + N_FORE].ob_spec.obspec.interiorcol = color_map[def_foreground];
option_tree[opt_pages[1] + N_BACK].ob_spec.obspec.interiorcol = color_map[def_background];
option_tree[opt_pages[1] + R_BACK].ob_spec.obspec.interiorcol = color_map[def_rforeground];
option_tree[opt_pages[1] + R_FORE].ob_spec.obspec.interiorcol = color_map[def_rbackground];
}

/* -----------------
   | Setup options |
   ----------------- */
void settings(void)
{
int obj;
extern int obj_colours;

set_dialog_settings();

SetOptPage(option_tree, opt_pages[0], OPTS1);

if (obj_colours < 16)
	DisableObj(option_tree, OPTS2);
else
	EnableObj(option_tree, OPTS2);

open_dialog(option_tree);
do
	{
	obj = do_dialog(option_tree, 0);
	switch(obj)
		{
		case OPTS1:		SetOptPage(option_tree, opt_pages[0], OPTS1);	break;
		case OPTS2:		SetOptPage(option_tree, opt_pages[1], OPTS2);	break;

		case SAVE:
		case OPT_OKAY:
			get_dialog_settings();
			if (obj == SAVE)
				save_config();
			break;

		default:
			{
			if (IsSelected(option_tree, OPTS2))	/* Color Page active ? */
				{
				switch(obj - opt_pages[1])
					{
					case N_FORE:
					case N_BACK:
					case R_BACK:
					case R_FORE:
						{
						int zw_col;
						
						zw_col = option_tree[obj].ob_spec.obspec.interiorcol;
						close_dialog();
						zw_col = select_color(zw_col);
						option_tree[obj].ob_spec.obspec.interiorcol = zw_col;
						open_dialog(option_tree);
						break;
						}
					}
				}
			} /* default */
		} /*	switch(obj) */

	}while(obj != OPT_OKAY && obj != OPT_CANC);
	
close_dialog();

if (obj == OPT_OKAY)
	{
	get_dialog_settings();
	}
}

/* -----------------------
   | Save current config |
   ----------------------- */
void save_config(void)
{
long fh;

fh = Fcreate("DRTELNET.CNF", 0);
if (fh > 0)
	{
	Fwrite((int)fh, 4, "TCNF");

	Fwrite((int)fh, sizeof(def_foreground), 	&def_foreground);
	Fwrite((int)fh, sizeof(def_background), 	&def_background);
	Fwrite((int)fh, sizeof(def_rforeground), 	&def_rforeground);
	Fwrite((int)fh, sizeof(def_rbackground), 	&def_rbackground);
	Fwrite((int)fh, sizeof(cut_128), 					&cut_128);
	Fwrite((int)fh, sizeof(monitor), 					&monitor);
	Fwrite((int)fh, sizeof(local_echo), 			&local_echo);
	Fwrite((int)fh, sizeof(key_mode), 				&key_mode);

	Fclose((int)fh);
	}
}

/* -----------------------
   | Save current config |
   ----------------------- */
void load_config(void)
{
long fh;
char xid[5];

fh = Fopen("DRTELNET.CNF", FO_READ);
if (fh > 0)
	{
	Fread((int)fh, 4, xid); xid[4] = 0;
	if (strcmp(xid, "TCNF"))
		{
		Fclose((int)fh);
		return;
		}
		
	Fread((int)fh, sizeof(def_foreground), 	&def_foreground);
	Fread((int)fh, sizeof(def_background), 	&def_background);
	Fread((int)fh, sizeof(def_rforeground), &def_rforeground);
	Fread((int)fh, sizeof(def_rbackground), &def_rbackground);
	Fread((int)fh, sizeof(cut_128), 				&cut_128);
	Fread((int)fh, sizeof(monitor), 				&monitor);
	Fread((int)fh, sizeof(local_echo), 			&local_echo);
	Fread((int)fh, sizeof(key_mode), 				&key_mode);
			
	Fclose((int)fh);
	}
}

/* -------------------
   | Color selection |
   ------------------- */
int select_color(int curr_color)
{
OBJECT *coltree;
int obj, i, color = curr_color;
static int c_ids[] = {COLS_1, COLS_2, COLS_3, COLS_4,
											COLS_5, COLS_6, COLS_7, COLS_8,
											COLS_9, COLS_10, COLS_11, COLS_12,
											COLS_13, COLS_14, COLS_15, COLS_16};
											
rsrc_gaddr(R_TREE, COLSET, &coltree);

for (i = 0; i < 16; i++)
	coltree[c_ids[i]].ob_spec.obspec.framesize = -1;
coltree[c_ids[curr_color]].ob_spec.obspec.framesize = -3;
	
open_dialog(coltree);
do
	{
	obj = do_dialog(coltree, 0) & 0x7FFF;
	
	switch(obj)
		{
		case COLS_1:
		case COLS_2:
		case COLS_3:
		case COLS_4:
		case COLS_5:
		case COLS_6:
		case COLS_7:
		case COLS_8:
		case COLS_9:
		case COLS_10:
		case COLS_11:
		case COLS_12:
		case COLS_13:
		case COLS_14:
		case COLS_15:
		case COLS_16:
			{
			for (i = 0; i < 16; i++)
				{
				if (coltree[c_ids[i]].ob_spec.obspec.framesize != -1)
					{
					coltree[c_ids[i]].ob_spec.obspec.framesize = -1;
					redraw_object(coltree, c_ids[i]);
					}
					
				if (c_ids[i] == obj)
					color = i;
				}
				
			coltree[obj].ob_spec.obspec.framesize = -3;
			redraw_object(coltree, COL_BKG);
			}			
		}
		
	}while(obj != COLS_OK && obj != COLS_CNC);
close_dialog();

if (obj == COLS_OK)
	return color;
else
	return curr_color;
}
