/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */
 
#include "extern.h"
#include "telnet.h"

int alive = FALSE;
unsigned char receive_buffer[256];
int in_buffer = 0, buf_pointer = 0;

int s = 0;
static init_flg = FALSE;

/* --------------------
   | Close connection |
   -------------------- */
void close_tcpip(void)
{
if (s)
  {
  close_socket(s);
  s = 0;
  alive = FALSE;
  }
menu_ienable(menu_adr, M_CONN, TRUE);  
menu_ienable(menu_adr, M_DCONN, FALSE);  
crt_exit();
init_flg = FALSE;
}

/* --------------
   | Re-Connect |
   -------------- */
void reconnect(int flg)
{
close_tcpip();
if (flg)
	if (!connect_to())
  	return;

start_telnet();
while (!alive)
  {
  if (!connect_to())
    return;
  start_telnet();
  }
init_flg = TRUE;
}

/* --------------------------------
   | Initialize TCP/IP connection |
   -------------------------------- */
void init_tcpip(void)
{
if (init_flg)
  {
  stop_telnet();
  init_flg = FALSE;
  }

alive = FALSE;
if (!connect_to())
  return;

if (!check_connection(0x4173))
	return;

start_telnet();
while (!alive)
  {
  if (!connect_to())
    return;
  start_telnet();
  }
init_flg = TRUE;
}

/* ---------------------------------------------------
   | Start up connection to telnet service on server |
   --------------------------------------------------- */
void start_telnet(void)
{
struct sockaddr_in remote;
struct hostent *host;
struct servent *serv;
unsigned long addr;
int port = 0;
char *str;

str = strchr(hostname, ':');
if (str)
	{
	*str = 0;
	port = atoi(str + 1);
	}

if (isdigit(hostname[0]))
  {
  addr = inet_addr(hostname);
  remote.sin_family = AF_INET;
  memcpy(&remote.sin_addr, &addr, sizeof(long));
  }
else
  {
  host = gethostbyname(hostname);
  if (!host)
    {
    form_alert(1, unknown_host);
    return;
    }
  remote.sin_family = host->h_addrtype;
  memcpy(&remote.sin_addr, host->h_addr, host->h_length);
  }

if (!port)
	{
	serv = getservbyname("telnet", NULL);
	if (!serv)
  	{
	  form_alert(1, unknown_service);
  	return;
  	}
	remote.sin_port = serv->s_port;
	}
else
	remote.sin_port = port;

if (str) *str = ':';

s = socket(AF_INET, SOCK_STREAM, 0);

if (!connect(s, (struct sockaddr *)&remote, (int)sizeof(remote)))
  {
	menu_ienable(menu_adr, M_CONN, FALSE);  
	menu_ienable(menu_adr, M_DCONN, TRUE);  
	crt_init();
  alive = TRUE;
	add_server(hostname);
  }
else
  {
  form_alert(1, conn_closed);							/* Cannot connect */
  close_tcpip();
  }
}

/* -----------------------
   | Stop telnet service | 
   ----------------------- */
void stop_telnet(void)
{
close_tcpip();
}

/* --------------------
   | Buffered receive |
   -------------------- */
int b_recv(int s, char *a, int flg)
{
int e;

if (buf_pointer < in_buffer){
  *a = receive_buffer[buf_pointer];
 }
else
  {
  if (flg == MSG_PEEK)
    {
    
#ifdef MINT_GW
	long prev;
	/* The following call to recv blocks under Mint-Net */
	/* (maybe MSG_PEEK is intended to be non-blocking */
	/* under draconis TCP/IP stack?) */ 
	/* Therefore we set it to non-blocking first. */
    prev = sockfcntl(s, SO_GETFL, SO_O_NONBLOCK);
    sockfcntl(s, SO_SETFL, SO_O_NONBLOCK);
#endif
    e = (int)recv(s, a, 1, MSG_PEEK);
#ifdef MINT_GW
    if( e == -1 ) {
    	e = 0;
    }
    sockfcntl(s, SO_SETFL, prev);
#endif
    return e;
    }
  else
    {
    in_buffer = (int)recv(s, receive_buffer, 256, 0);
    if (in_buffer < 0)
      {
      form_alert(1, conn_closed);
      close_tcpip();
      return 0;
      }

    buf_pointer = 0;
    *a = receive_buffer[0];
    }
  }

if (flg != MSG_PEEK)
  {
  buf_pointer++;

  if (*a == '\r')
    {
    char x;

    if (b_recv(s, &x, MSG_PEEK) == 1 && !x)
      b_recv(s, &x, 0);
    }
  else
    {
    static pass = 0;

    if (*(unsigned char *)a == 128 && !pass && cut_128)
      {
      unsigned char x;

      pass = 1;
      if (b_recv(s, (char *)&x, MSG_PEEK) == 1 && x == 128)
        b_recv(s, (char *)&x, 0);
      pass = 0;
      }
    }
  }
return 1;
}

/* ---------------------------------------
   | Check input queue for incoming data |
   --------------------------------------- */
int tcpip_state(void)
{
unsigned char c;
int x;

if (!init_flg)
	return FALSE;
	
if ((x = b_recv(s, (char *)&c, MSG_PEEK)) == 1)
  if (c == IAC)
    {
    recv_data();
    return FALSE;
    }
  else
    return TRUE;
else
  {
  if (x < 0)
  	{
    form_alert(1, conn_closed);
    close_tcpip();
    }
  return FALSE;
  }
}

/* -------------------------------
   | Are there any incoming data |
   -------------------------------  */
int tcpip_read(char *chr)
{
int x;

x = recv_data();
if (x == -1)
  return 0;

*chr = (char)x;
return 1;
}

/* -------------------
   | Write to TCP/IP |
   ------------------- */
void tcpip_write(char chr)
{
send(s, &chr, 1, 0);
}

/* ----------------------------------------
   | Send 'Please you use' option to host |
   ---------------------------------------- */
void send_do_cmd(unsigned char option)
{
static unsigned char do_iac = IAC;
static unsigned char do_cmd = DO;

send(s, &do_iac, 1, 0);
send(s, &do_cmd, 1, 0);
send(s, &option, 1, 0);
}

/* ----------------------------------------
   | Send 'Please you use' option to host |
   ---------------------------------------- */
void send_wont_cmd(unsigned char option)
{
static unsigned char wont_iac = IAC;
static unsigned char wont_cmd = WONT;

send(s, &wont_iac, 1, 0);
send(s, &wont_cmd, 1, 0);
send(s, &option, 1, 0);
}

/* ------------------------------------
   | Send 'I will use' option to host |
   ------------------------------------ */
void send_will_cmd(unsigned char option)
{
static unsigned char will_iac = IAC;
static unsigned char will_cmd = WILL;

send(s, &will_iac, 1, 0);
send(s, &will_cmd, 1, 0);
send(s, &option, 1, 0);
}

/* -----------------------
   | Send subnegotiation |
   ----------------------- */
void send_subnegotiation(unsigned char option, char *sub)
{
static unsigned char sub_iac = IAC;
static unsigned char sub_cmd = SB;
static unsigned char sub_is = 0;
static unsigned char sub_ias = 0xFF;
static unsigned char sub_se = SE;

send(s, &sub_iac, 1, 0);
send(s, &sub_cmd, 1, 0);
send(s, &option, 1, 0);
send(s, &sub_is, 1, 0);
send(s, sub, strlen(sub), 0);
send(s, &sub_ias, 1, 0);
send(s, &sub_se, 1, 0);
}

/* --------------------------------
   | Terminal type subnegotiation |
   -------------------------------- */
void handle_ttype(unsigned char *cmd_buf)
{
if (cmd_buf[1] == TELQUAL_SEND)
  send_subnegotiation(TELOPT_TTYPE, "ansi");
}

/* -----------------------------------
   | Display location subnegotiation |
   ----------------------------------- */
void handle_xdisploc(unsigned char *cmd_buf)
{
if (cmd_buf[1] == TELQUAL_SEND)
  send_subnegotiation(TELOPT_XDISPLOC, "\0");
}

/* -------------------------
   | Terminal type command |
   ------------------------- */
void ttype_cmd(unsigned char cmd)
{
switch(cmd)
  {
  case DO:    send_will_cmd(TELOPT_TTYPE);    break;
  case DONT:
  case WILL:
  case WONT:
    break;
  }
}

/* ----------------------------
   | display location command |
   ---------------------------- */
void xdisploc_cmd(unsigned char cmd)
{
switch(cmd)
  {
  case DO:    send_will_cmd(TELOPT_XDISPLOC);  break;
  case DONT:
  case WILL:
  case WONT:
    break;
  }
}

/* -------------------------------
   | Receive an incoming command |
   ------------------------------- */
void receive_command(void)
{
unsigned char cmd_buf[200], cmd, opt;
int i;

b_recv(s, (char *)&cmd, 0);
switch(cmd)
  {
  case SB:
    i = 0;
    do
      {
      b_recv(s, (char *)(cmd_buf + i), 0);
      }while(cmd_buf[i++] != SE);
    
    switch(cmd_buf[0])
      {
      case TELOPT_TTYPE:    handle_ttype(cmd_buf); break;
      case TELOPT_XDISPLOC: handle_xdisploc(cmd_buf); break;
      default:
        if (cmd == DO)
          send_wont_cmd(cmd_buf[0]);
      }
    break;

  case DONT:
  case DO:
  case WONT:
  case WILL:
    b_recv(s, (char *)&opt, 0);
    switch(opt)
      {
      case TELOPT_TTYPE:      ttype_cmd(cmd); break;
      case TELOPT_XDISPLOC:   xdisploc_cmd(cmd); break;
      default:
        switch(cmd)
          {
          case DO:    send_wont_cmd(opt); break;
          case DONT:  break;
          case WILL:  send_do_cmd(opt); break;
          case WONT:  break;
          }
      }
    break;

  case GA:
    break;

  case EL:
    break;

  case EC:
    break;

  case AYT:  
    break;
    
  case AO:
    break;
    
  case IP:
    break;
    
  case BREAK:
    break;
    
  case DM:
    break;
    
  case NOP:
    break;    

  case EOR:
    break;  
  }  
}

/* ------------------------------------------------
   | Receive incoming data, if data is available. |
   | If a IAC is send receive the command.        |
   ------------------------------------------------ */
int recv_data(void)
{
unsigned char c;
int i;

i = b_recv(s, (char *)&c, 0);
if (i == 1 && c)
  {
  if (c == IAC)
    {
    receive_command();
    return -1;
    }
  else
    return c;
  }
return 0;
}

