/*
 * Copyright (c) Draconis Internet Package Project
 * Jens Heitmann <jh_draconis@users.sourceforge.net>
 * 
 * This software is licenced under the GNU Public License.
 * Please see COPYING for further information.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110, USA.
 * or have a look to http://www.gnu.org.
 * 
 * Author: Jens Heitmann
 * Version-Date: 2007-08-01
 * 
 */

#include "extern.h"

#define FMD_DEFLT 		0
#define FMD_FORWARD 	1
#define FMD_BACKWARD 	2

extern int icony_flg;

int find_obj(OBJECT *tree, int start_obj, int which)
{
int obj, flag, theflag, inc;

obj = 0;
flag = EDITABLE;
inc = 1;
switch(which)
	{
	case FMD_BACKWARD:
		inc = -1;
	case FMD_FORWARD:
		obj = start_obj + inc;
		break;
	case FMD_DEFLT:
		flag = DEFAULT;
		break;
	}

while (obj >= 0)
	{
	theflag = tree[obj].ob_flags;
	if (theflag & flag)
		if (!(tree[obj].ob_state & DISABLED))
			return obj;
	if (theflag & LASTOB)
		obj = -1;
	else
		obj += inc;
	}
return start_obj;
}

int fm_inifld(OBJECT *tree, int start_fld)
{
if (!start_fld)
	start_fld = find_obj(tree, 0, FMD_FORWARD);
return start_fld;
}

/* --------------------------
   | Check for mnemonic key |
   -------------------------- */
int check_key(char *s, char chr)
{
do
	{
	s = strchr(s, '&');
	if (s)
		{
		if (s[1] == '&') 
			s += 2;
		else						
			if (toupper(s[1]) == chr)
				return 1;
			else
				s++;
		}
	}while(s);

return 0;
}

/* --------------------------------
   | Search for keyboard mnemonic |
   -------------------------------- */
int find_key(OBJECT *tree, int key)
{
int i, flags, prev;
char chr;
KEYTAB *k;

k = Keytbl((char *)-1L, (char *)-1L, (char *)-1L);
chr = k->shift[key >> 8];

i = 0;
while (i != -1)
	{
	flags = tree[i].ob_flags;
	if (!(tree[i].ob_state & DISABLED))
		{
		switch(tree[i].ob_type)
			{
			case (13<<8)|G_FTEXT:
			case (14<<8)|G_FTEXT:
				if (check_key(tree[i].ob_spec.tedinfo->te_ptmplt, chr))
					return i;
			case (15<<8)|G_BUTTON:
			case (40<<8)|G_STRING:
				if (check_key(tree[i].ob_spec.free_string, chr))
					return i;
				break;
			}
		}
	
	if (flags & LASTOB)
		break;	

	if (tree[i].ob_head != -1)
		i = tree[i].ob_head;
	else
		{
		do
			{
			prev = i;
			i = tree[i].ob_next;
			}while(i != -1 && prev == tree[i].ob_tail);
		}
	}
return -1;
}

int x_form_do(OBJECT *tree, int start_fld)
{
int edit_obj;
int next_obj;
int which, cont;
int idx, msg[8], top;
int mx, my, mb, ks, kr, br;

next_obj = fm_inifld(tree, start_fld);
edit_obj = 0;
cont = TRUE;
while(cont)
	{
	wind_get(ROOT, WF_TOP, &top);
	
	if (top == dialog_wind)
		if (!icony_flg)
			if ((next_obj != 0) && (edit_obj != next_obj))
				{
				edit_obj = next_obj;
				next_obj = 0;
				objc_edit(tree, edit_obj, 0, &idx, ED_INIT);
				}
		
	which = evnt_multi(MU_MESAG|MU_KEYBD|MU_BUTTON, 2, 1, 1,
										 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
										 msg, 0, 0, &mx, &my, &mb, &ks, &kr, &br);
										 
	if (top == dialog_wind && !icony_flg)
		{
		if (which & MU_KEYBD)
			{
			if (ks & 8)
				{
				next_obj = find_key(tree, kr);
				if (next_obj != -1)
					{
					if (tree[next_obj].ob_type == ((40<<8)|G_STRING))
						next_obj--;
						
					set_special_objects(tree);
					cont = form_button(tree, next_obj, 1, &next_obj);
					release_special_objects(tree);
					kr = 0;
					}
				else
					next_obj = 0;
				}

			if (kr)
				{
				set_special_objects(tree);
				cont = form_keybd(tree, edit_obj, next_obj, kr, &next_obj, &kr);
				release_special_objects(tree);
				if (kr)
					{
					tree[edit_obj].ob_flags |= HIDETREE;
					objc_edit(tree, edit_obj, kr, &idx, ED_CHAR);
					tree[edit_obj].ob_flags &= ~HIDETREE;
	
					objc_edit(tree, edit_obj, 0, &idx, ED_END);
					redraw_object(tree, edit_obj);
					objc_edit(tree, edit_obj, 0, &idx, ED_END);
					}
				}
			}
	
		if (which & MU_BUTTON)
			{
			next_obj = objc_find(tree, ROOT, MAX_DEPTH, mx, my);
			if (next_obj == -1)
				{
				Cconws("\a");
				next_obj = 0;
				}
					
			if (tree[next_obj].ob_type == ((40<<8)|G_STRING))
				next_obj--;
				
			set_special_objects(tree);
			cont = form_button(tree, next_obj, br, &next_obj);
			release_special_objects(tree);
			}
		}
			
	if (which & MU_MESAG && msg[3] == dialog_wind)
		{
		if (!icony_flg && top == dialog_wind)
			objc_edit(tree, edit_obj, 0, &idx, ED_END);

		dialog_message(tree, msg);
		wind_get(ROOT, WF_TOP, &top);

		if (!icony_flg && top == dialog_wind)
			objc_edit(tree, edit_obj, 0, &idx, ED_END);
		}
	
	if (top == dialog_wind)
		if ((!cont) || ((next_obj != 0) && (next_obj != edit_obj)) ||
				(icony_flg && edit_obj))
			{
			if (!icony_flg)
				objc_edit(tree, edit_obj, 0, &idx, ED_END);

			if ((tree[next_obj].ob_state & DISABLED) || icony_flg)
				next_obj = 0;
			}
	}

return next_obj;
}

